#include "juego.h"

Juego::Juego()
{
}

Juego::~Juego()
{
}

int
Juego::aleatorio(int min, int max)
{
	return ( rand() % (max-min+1) + min );
}

int
Juego::suma(int a, int b)
{
	return a+b;
}

int
Juego::resta(int a, int b)
{
	return a-b;
}

int 
Juego::multiplicacion(int a, int b)
{
	return a*b;
}

int 
Juego::division(int a, int b)
{
	return a/b;
}

void
Juego::numerosValidos(int &a, int &b, int &operacion, int nivel)
{
	bool valido = true;
	
	switch(nivel)
	{
		case 1:
		case 2: operacion = 0; break;
		case 3: 
		case 4: operacion = aleatorio(0,1); break;
		case 5:
		case 6: operacion = aleatorio(0,2); break;
		case 7:
		case 8:
		case 9: operacion = aleatorio(0,3); break;
	}
	
	do
	{
		a = aleatorio(0,nivel*5);
		b = aleatorio(0,nivel*5);
		
		valido = true;
		
		switch (operacion)
		{
			case 0: break;
			case 1: if(a-b < 0) valido = false; break;
			case 2: if(a*b > nivel*5) valido = false; break;
			case 3: if(b == 0 or a%b != 0) valido = false; break;
		}
	}
	while(!valido);
}

int
Juego::tiempoTranscurrido(long inicio)
{
	struct timeval actual;
	gettimeofday(&actual,NULL);
	return (actual.tv_sec - inicio);
}

int
Juego::ejecutar(Datos &datos, Challenges &challenges)
{
	int x, y, resultado, aciertos = 0, tiempoPartida = datos.getTiempoPartida(), operacion = 0;
	int racha = 0, nivel = datos.getNivel(), puntuacion = 0, fallos = 0, total = 0;
	struct timeval inicio;
	string cadena;
	long segundos = 0;

	int (Juego::*funcion[])(int, int) = { 
		&Juego::suma, 
		&Juego::resta, 
		&Juego::multiplicacion, 
		&Juego::division };

	char signos[] = {'+','-','*','/'};
	
	gettimeofday(&inicio,NULL);

	srand(time(NULL));

	do	
	{
		numerosValidos(x, y, operacion, nivel);
	
		segundos = tiempoTranscurrido(inicio.tv_sec);
		cout << "[" << segundos << "/" << tiempoPartida << "|A:" << aciertos << "|N:" << nivel << "|P:" << puntuacion << "] "; 
		cout << x << signos[operacion] << y << "=";
		getline(cin, cadena);
		if(cadena == "")
			resultado = -1;
		else
			resultado = atoi(cadena.c_str());
		if(tiempoTranscurrido(inicio.tv_sec) < tiempoPartida)
		{
			if( (this->*funcion[operacion])(x,y) == resultado )
			{
				cout << "ACERTASTE" << endl;
				racha++;
				if(racha >= datos.getSubidaNivel())
				{
					if(nivel < 8)
					{
						cout << "¡Sube de nivel!" << endl;
						nivel ++;
						fallos = 0;
					}
					else if (nivel == 8)
					{
						cout << "MAXIMUN LEVEL! Unleash your fury" << endl;
						nivel++;
						fallos = 0;
					}
					racha = 0;
				}
				aciertos++;
				puntuacion += (nivel+1)/2;
			}
			else
			{
				cout << "FALLASTE, La respuesta correcta era " << (this->*funcion[operacion])(x,y) << endl;
				fallos++;
				racha = 0;
				if(fallos >= datos.getBajadaNivel())
				{
					fallos = 0;
					if(nivel > 1)
					{
						cout << "¡Baja de nivel!" << endl;
						nivel--;
					}
				}
			}
			total++;
		}
	}
	while(tiempoTranscurrido(inicio.tv_sec) < tiempoPartida);
	
	cout << endl << "El tiempo se ha agotado" << endl << endl;
	cout << "ESTADISTICAS" << endl;
	cout << "------------" << endl;
	cout << "Aciertos: " << aciertos << "/" << total << endl;
	cout << "Nivel: " << nivel << endl;
	cout << "Puntuación: " << puntuacion << endl << endl;
	
	challenges.ejecutar(datos, aciertos, total, nivel, puntuacion);
	
	return puntuacion;
}
